package dev.ninjdai.werewolf.uhc.effects;

import net.minestom.server.potion.Potion;
import org.jetbrains.annotations.Nullable;

import java.util.Optional;

public class EffectProperty {
    public EffectProperty(Potion potion, PlayerProperties properties, int duration, int activationTime){
        this.potion = Optional.ofNullable(potion);
        this.properties = Optional.ofNullable(properties);
        this.timeData = Optional.of(new TimeData(duration, activationTime));
    }

    EffectProperty(Builder builder){
        this.potion = builder.potion;
        this.properties = builder.properties;
        this.timeData = builder.timeData;
    }

    Optional<Potion> potion;
    Optional<PlayerProperties> properties;
    /**
     * The duration and time of activation for the effect. If not present, effect is constantly active.
     */
    Optional<TimeData> timeData;

    public record TimeData (int duration, int activationTime) {}

    public static class Builder {
        private Optional<Potion> potion;
        private Optional<PlayerProperties> properties;
        private Optional<TimeData> timeData;

        public Builder setPotion(Potion potion){
            this.potion = Optional.of(potion);
            return this;
        }

        public Builder setProperties(PlayerProperties properties){
            this.properties = Optional.of(properties);
            return this;
        }

        public Builder setTimeData(TimeData timeData){
            this.timeData = Optional.of(timeData);
            return this;
        }

        public EffectProperty build(){
            return new EffectProperty(this);
        }
    }
}
